<img src="http://t-neumann.github.io/slamdunk/images/slamdunk_logo_light.png" width="300" title="Slamdunk">

### Streamlining SLAM-Seq analysis with ultra-high sensitivity.

[![GitHub release](https://img.shields.io/github/release/t-neumann/slamdunk.svg)](https://github.com/t-neumann/slamdunk/releases/latest)
[![Travis CI](https://img.shields.io/travis/t-neumann/slamdunk.svg)](https://travis-ci.org/t-neumann/slamdunk)

[![Docker Pulls](https://img.shields.io/docker/pulls/tobneu/slamdunk.svg)](https://hub.docker.com/r/tobneu/slamdunk)
[![Docker Automated build](https://img.shields.io/docker/automated/tobneu/slamdunk.svg)](https://hub.docker.com/r/tobneu/slamdunk/builds/)

[![install with bioconda](https://img.shields.io/badge/install%20with-bioconda-brightgreen.svg?style=flat-square)](http://bioconda.github.io/recipes/slamdunk/README.html)
[![Anaconda build](https://anaconda.org/bioconda/slamdunk/badges/version.svg
)](https://anaconda.org/bioconda/slamdunk)
[![Anaconda downloads](https://anaconda.org/bioconda/slamdunk/badges/downloads.svg
)](https://anaconda.org/bioconda/slamdunk)

[![PyPI release](https://img.shields.io/pypi/v/slamdunk.svg)](https://pypi.python.org/pypi/slamdunk)
![Github Stars](https://img.shields.io/github/stars/t-neumann/slamdunk.svg?style=social&label=Star)

-----

### Slamdunk documentation

http://t-neumann.github.io/slamdunk

### Please cite

Neumann, T., Herzog, V. A., Muhar, M., Haeseler, von, A., Zuber, J., Ameres, S. L., & Rescheneder, P. (2019). [Quantification of experimentally induced nucleotide conversions in high-throughput sequencing datasets](https://bmcbioinformatics.biomedcentral.com/articles/10.1186/s12859-019-2849-7). BMC Bioinformatics, 20(1), 258. http://doi.org/10.1186/s12859-019-2849-7

